/*
 * PHP Reformatter - A tool to make dirty PHP code readable.
 * Copyright (C) 2007 Joris van Rooij <jorrizza@jrrzz.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "codeblock.h"

char *tags[] = {
  "PHP_START_TAG",
  "PHP_END_TAG",
  "PHP_COMMENT_SINGLE",
  "PHP_COMMENT_MULTI",
  "PHP_TYPECAST_INT",
  "PHP_TYPECAST_FLOAT",
  "PHP_TYPECAST_STRING",
  "PHP_TYPECAST_ARRAY",
  "PHP_TYPECAST_OBJECT",
  "PHP_FUNCTION_TYPE",
  "PHP_FUNCTION_NAME",
  "PHP_FUNCTION_BODY_START",
  "PHP_FUNCTION_BODY_END",
  "PHP_FUNCTION_BODY_SEPERATOR",
  "PHP_FUNCTION_START",
  "PHP_FUNCTION_END",
  "PHP_LOOP_WHILE",
  "PHP_LOOP_FOREACH",
  "PHP_LOOP_FOR",
  "PHP_LOOP_DO",
  "PHP_LOOP_BODY_START",
  "PHP_LOOP_BODY_END",
  "PHP_LOOP_BODY_SEPERATOR",
  "PHP_LOOP_START",
  "PHP_LOOP_END",
  "PHP_SWITCH_TYPE",
  "PHP_SWITCH_BODY_START",
  "PHP_SWITCH_BODY_END",
  "PHP_SWITCH_START",
  "PHP_SWITCH_END",
  "PHP_SWITCH_CASE",
  "PHP_OOP_CLASS_TYPE",
  "PHP_OOP_CLASS_NAME",
  "PHP_OOP_CLASS_START",
  "PHP_OOP_CLASS_END",
  "PHP_OOP_ARROW",
  "PHP_OOP_SCOPE_RESOLUTION",
  "PHP_ERROR_SILENCE",
  "PHP_INCREMENTOR",
  "PHP_DECREMENTOR",
  "PHP_ARITHMETIC_NEGATION",
  "PHP_ARITHMETIC_NOT",
  "PHP_REFERENCE",
  "PHP_ARRAY_ARROW",
  "PHP_STRING",
  "PHP_ARITHMETIC",
  "PHP_COMPARISON",
  "PHP_ASSIGNMENT",
  "PHP_ARITHMETIC_INV",
  "PHP_VARIABLE",
  "PHP_NUMBER",
  "PHP_ARRAY_TYPE",
  "PHP_ARRAY_START",
  "PHP_ARRAY_END",
  "PHP_ARRAY_SEPERATOR",
  "PHP_IF_TYPE",
  "PHP_IF_BODY_START",
  "PHP_IF_BODY_END",
  "PHP_IF_START",
  "PHP_IF_END",
  "PHP_ELSEIF_TYPE",
  "PHP_ELSE_TYPE",
  "PHP_KEY_START",
  "PHP_KEY_END",
  "PHP_SUBSTRING_START",
  "PHP_SUBSTRING_END",
  "PHP_CONCATENATOR",
  "PHP_LINE_TERMINATOR",
  "PHP_FUNCTION_CALL",
  "PHP_FUNCTION_CALL_BODY_START",
  "PHP_FUNCTION_CALL_BODY_END",
  "PHP_FUNCTION_CALL_BODY_SEPERATOR",
  "PHP_BOOLGROUP_START",
  "PHP_BOOLGROUP_END",
  "PHP_KEYWORD_BREAK",
  "PHP_KEYWORD_EXIT",
  "PHP_KEYWORD_DIE",
  "PHP_KEYWORD_ECHO",
  "PHP_KEYWORD_PRINT",
  "PHP_KEYWORD_REQUIREONCE",
  "PHP_KEYWORD_REQUIRE",
  "PHP_KEYWORD_INCLUDEONCE",
  "PHP_KEYWORD_INCLUDE",
  "PHP_KEYWORD_RETURN",
  "PHP_KEYWORD_CONTINUE",
  "PHP_KEYWORD_AS",
  "PHP_KEYWORD_GLOBAL",
  "PHP_KEYWORD_TRUE",
  "PHP_KEYWORD_FALSE",
  "PHP_KEYWORD_CASE",
  "PHP_KEYWORD_DEFAULT",
  "PHP_OOP_KEYWORD_NEW",
  "PHP_OOP_KEYWORD_EXTENDS",
  "PHP_OOP_KEYWORD_PUBLIC",
  "PHP_OOP_KEYWORD_PROTECTED",
  "PHP_OOP_KEYWORD_PRIVATE",
  "PHP_OOP_KEYWORD_VAR",
  "PHP_OOP_KEYWORD_CONST",
  "PHP_OOP_KEYWORD_STATIC",
  "PHP_SHORTHAND_IF",
  "PHP_SHORTHAND_ELSE",
  "PHP_DEFINED",
  "PHP_UNDEFINED",
  "PHP_MISC_SEPERATOR",
  "PHP_STDOUT_DATA"
};

void codeblock_init (struct codeblock_s **block, tag_t tag, char *content) {
  *block = (struct codeblock_s*) malloc(sizeof (struct codeblock_s));
  (*block)->content = (char *) malloc(strlen(content) + 1);
  strncpy((*block)->content, content, strlen(content));
  (*block)->content[strlen(content)] = '\0';
  (*block)->next = NULL;
  (*block)->tag = tag;
}

void codeblock_free (struct codeblock_s *block) {
  struct codeblock_s *tmp, *tmp2;

  tmp = block;
  while (tmp != NULL) {
    free(tmp->content);
    tmp2 = tmp->next;
    free(tmp);
    tmp = tmp2;
  }
}

void codeblock_debug (struct codeblock_s *block, char *filename) {
  unsigned int i = 0, j;

  fprintf(stderr, "Debugger: %s: Debugging enabled. Debugging code tree...\n", filename);
  while (block != NULL) {
    fprintf(stderr, "Debugger: %s: 0x%08X %-32s ", filename, i, tags[block->tag]);
    for (j = 0; j < strlen(block->content); j++) {
      if (block->content[j] == '\n')
        fprintf(stderr, "\\n");
      else if (block->content[j] == '\t')
        fprintf(stderr, "\\t");
      else
        fprintf(stderr, "%c", block->content[j]);
    }
    printf("\n");
    block = block->next;
    i++;
  }
  fprintf(stderr, "Debugger: %s: End.\n\n", filename);
}
